#!/usr/bin/env python3

"""
FastAPI app for sentiment analysis.
"""

import logging
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel
from fastapi.middleware.cors import CORSMiddleware
from opinion_miner_api.model import model

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

app = FastAPI()

# CORS (optional, useful for frontend)
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


class TextInput(BaseModel):
    text: str


@app.post("/predict")
async def predict_sentiment(input: TextInput):
    """
    Predict the sentiment of the given text.

    Args:
        input (TextInput): The input text for sentiment analysis.

    Returns:
        dict: The sentiment analysis result.
    """
    max_length = 1000  # Catch `internal server error 500` with upper boundary
    if len(input.text) > max_length:
        # Send 400: Bad Request instead of 500: Server Error
        raise HTTPException(status_code=400, detail=f"Text too long. Maximum length is {max_length} characters.")

    logger.debug(f"Received text for prediction: {input.text}")
    prediction = model.predict(input.text)
    logger.debug(f"Prediction result: {prediction}")
    return {"result": prediction[0]}


@app.get("/")
def read_root():
    return {"message": "Welcome to the AI Opinion Miner Cloud API."}
