#!/usr/bin/env python3

"""
Load singleton Sentiment Model.
"""

import logging
from functools import lru_cache
from typing import Optional, List, Dict
import csv
import torch
import urllib.request
import numpy as np
from scipy.special import softmax
from transformers import AutoModelForSequenceClassification, AutoTokenizer

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class SentimentModel:
    def __init__(self, model_name: Optional[str], mapping_link: Optional[str] = None):
        self.model_name = model_name or ""
        self.tokenizer: Optional[AutoTokenizer] = None
        self.model: Optional[AutoModelForSequenceClassification] = None
        self.labels: List[str] = []
        if mapping_link is not None:
            self.mapping_link = mapping_link
            self._load_label_mapping(self.mapping_link)

    def _load_label_mapping(self, mapping_link: str) -> None:
        """Loads label mapping from the model's GitHub repository."""
        try:
            logger.info("Loading label mapping...")
            with urllib.request.urlopen(mapping_link) as f:
                html = f.read().decode("utf-8").split("\n")
                csvreader = csv.reader(html, delimiter="\t")
                self.labels = [row[1] for row in csvreader if len(row) > 1]
            logger.info(f"Label mapping loaded: {self.labels}")
        except Exception as e:
            logger.error(f"Failed to load label mapping: {e}")
            raise

    def _preprocess(self, text: str) -> str:
        """Preprocesses text by replacing usernames and links."""
        new_text = []
        for t in text.split(" "):
            t = "@user" if t.startswith("@") and len(t) > 1 else t
            t = "http" if t.startswith("http") else t
            new_text.append(t)
        return " ".join(new_text)

    def load_model(self) -> None:
        """Loads the sentiment analysis model and tokenizer if not already loaded."""
        if self.model is None or self.tokenizer is None:
            try:
                logger.info(f"Loading model and tokenizer: {self.model_name}")
                self.tokenizer = AutoTokenizer.from_pretrained(self.model_name)
                self.model = AutoModelForSequenceClassification.from_pretrained(
                    self.model_name
                )
                logger.info("Model and tokenizer loaded successfully.")
            except Exception as e:
                logger.error(f"Failed to load model: {e}")
                raise

    def predict(self, text: str) -> List[Dict[str, str]]:
        """Predicts sentiment of the given text.

        Args:
            text (str): The text to analyze.

        Returns:
            List[Dict[str, str]]: The sentiment analysis results with labels and scores.
        """
        self.load_model()
        try:
            logger.info(f"Predicting sentiment for text: {text}")
            text = self._preprocess(text)
            encoded_input = self.tokenizer(text, return_tensors="pt")
            with torch.no_grad():
                output = self.model(**encoded_input)
            scores = output.logits[0].detach().numpy()
            scores = softmax(scores)

            # Map scores to labels
            results = []
            ranking = np.argsort(scores)[::-1]
            for i in range(scores.shape[0]):
                label = self.labels[ranking[i]]
                score = float(scores[ranking[i]])
                results.append({"label": label, "score": np.round(score, 4)})

            logger.info(f"Prediction result: {results}")
            return results
        except Exception as e:
            logger.error(f"Prediction failed: {e}")
            raise


@lru_cache(maxsize=1)  # Ensures singleton behavior
def get_model(
    model: str = "cardiffnlp/twitter-roberta-base-sentiment",
    mapping_link: Optional[str] = "https://raw.githubusercontent.com/cardiffnlp/tweeteval/main/datasets/sentiment/mapping.txt",
) -> SentimentModel:
    return SentimentModel(model, mapping_link)


model = get_model()
